@@ -170,6 +170,13 @@ typedef enum _snd_ctl_event_type {
 #define SND_CTL_TLVT_CONTAINER		0x0000
 /** TLV type - basic dB scale */
 #define SND_CTL_TLVT_DB_SCALE		0x0001
+/** TLV type - linear volume */
+#define SND_CTL_TLVT_DB_LINEAR		0x0002
+/** TLV type - dB range container */
+#define SND_CTL_TLVT_DB_RANGE		0x0003
+
+/** Mute state */
+#define SND_CTL_TLV_DB_GAIN_MUTE	-9999999
 
 /** CTL type */
 typedef enum _snd_ctl_type {
@@ -207,6 +214,10 @@ int snd_card_get_index(const char *name);
 int snd_card_get_name(int card, char **name);
 int snd_card_get_longname(int card, char **name);
 
+int snd_device_name_hint(int card, const char *iface, void ***hints);
+int snd_device_name_free_hint(void **hints);
+char *snd_device_name_get_hint(const void *hint, const char *id);
+
 int snd_ctl_open(snd_ctl_t **ctl, const char *name, int mode);
 int snd_ctl_open_lconf(snd_ctl_t **ctl, const char *name, int mode, snd_config_t *lconf);
 int snd_ctl_close(snd_ctl_t *ctl);
