@@ -157,6 +157,7 @@ int _snd_pcm_route_open(snd_pcm_t **pcmp, const char *name,
  */
 int snd_pcm_rate_open(snd_pcm_t **pcmp, const char *name,
 		      snd_pcm_format_t sformat, unsigned int srate,
+		      const snd_config_t *converter,
 		      snd_pcm_t *slave, int close_slave);
 int _snd_pcm_rate_open(snd_pcm_t **pcmp, const char *name,
 		       snd_config_t *root, snd_config_t *conf,
@@ -176,6 +177,7 @@ int _snd_pcm_hooks_open(snd_pcm_t **pcmp, const char *name,
  */
 int snd_pcm_ladspa_open(snd_pcm_t **pcmp, const char *name,
 			const char *ladspa_path,
+			unsigned int channels,
 			snd_config_t *ladspa_pplugins,
 			snd_config_t *ladspa_cplugins,
 			snd_pcm_t *slave, int close_slave);
