@@ -1,5 +1,5 @@
 /**
- * \file <alsa/rawmidi.h>
+ * \file include/rawmidi.h
  * \brief Application interface library for the ALSA driver
  * \author Jaroslav Kysela <perex@suse.cz>
  * \author Abramo Bagnara <abramo@alsa-project.org>
@@ -7,8 +7,8 @@
  * \date 1998-2001
  *
  * Application interface library for the ALSA driver
- *
- *
+ */
+/*
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Lesser General Public License as
  *   published by the Free Software Foundation; either version 2.1 of
@@ -74,7 +74,9 @@ typedef enum _snd_rawmidi_type {
 	/** Shared memory client RawMidi (not yet implemented) */
 	SND_RAWMIDI_TYPE_SHM,
 	/** INET client RawMidi (not yet implemented) */
-	SND_RAWMIDI_TYPE_INET
+	SND_RAWMIDI_TYPE_INET,
+	/** Virtual (sequencer) RawMidi */
+	SND_RAWMIDI_TYPE_VIRTUAL
 } snd_rawmidi_type_t;
 
 int snd_rawmidi_open(snd_rawmidi_t **in_rmidi, snd_rawmidi_t **out_rmidi,
