@@ -415,12 +415,13 @@ beginning:</P>
 
 
 #include <stdarg.h>
-#include <dlfcn.h>
 #include <limits.h>
 #include <sys/stat.h>
-#include <pthread.h>
 #include <locale.h>
 #include "local.h"
+#ifdef HAVE_LIBPTHREAD
+#include <pthread.h>
+#endif
 
 #ifndef DOC_HIDDEN
 
@@ -470,7 +471,7 @@ static int safe_strtoll(const char *str, long long *val)
 	if (!*str)
 		return -EINVAL;
 	errno = 0;
-	if (sscanf(str, "%Ld%n", &v, &endidx) < 1)
+	if (sscanf(str, "%Li%n", &v, &endidx) < 1)
 		return -EINVAL;
 	if (str[endidx])
 		return -EINVAL;
@@ -579,12 +580,12 @@ static int get_char_skip_comments(input_t *input)
 			if (err < 0)
 				return err;
 			if (!strncmp(str, "confdir:", 8)) {
-				char *tmp = malloc(strlen(DATADIR "/alsa") + 1 + strlen(str + 8) + 1);
+				char *tmp = malloc(strlen(ALSA_CONFIG_DIR) + 1 + strlen(str + 8) + 1);
 				if (tmp == NULL) {
 					free(str);
 					return -ENOMEM;
 				}
-				sprintf(tmp, DATADIR "/alsa/%s", str + 8);
+				sprintf(tmp, ALSA_CONFIG_DIR "/%s", str + 8);
 				free(str);
 				str = tmp;
 			}
@@ -2404,9 +2405,16 @@ int snd_config_save(snd_config_t *config, snd_output_t *out)
 #define SND_CONFIG_SEARCH_ALIAS(config, base, key, result, fcn1, fcn2) \
 { \
 	snd_config_t *res = NULL; \
-	int err, first = 1; \
+	char *old_key; \
+	int err, first = 1, maxloop = 1000; \
 	assert(config && key); \
-	do { \
+	while (1) { \
+		old_key = strdup(key); \
+		if (old_key == NULL) { \
+			err = -ENOMEM; \
+			res = NULL; \
+			break; \
+		} \
 		err = first && base ? -EIO : fcn1(config, config, key, &res); \
 		if (err < 0) { \
 			if (!base) \
@@ -2415,8 +2423,22 @@ int snd_config_save(snd_config_t *config, snd_output_t *out)
 			if (err < 0) \
 				break; \
 		} \
+		if (snd_config_get_string(res, &key) < 0) \
+			break; \
+		if (!first && (strcmp(key, old_key) == 0 || maxloop <= 0)) { \
+			if (maxloop == 0) \
+				SNDERR("maximum loop count reached (circular configuration?)"); \
+			else \
+				SNDERR("key %s refers to itself", key); \
+			err = -EINVAL; \
+			res = NULL; \
+			break; \
+		} \
+		free(old_key); \
 		first = 0; \
-	} while (snd_config_get_string(res, &key) >= 0); \
+		maxloop--; \
+	} \
+	free(old_key); \
 	if (!res) \
 		return err; \
 	if (result) \
@@ -2584,7 +2606,7 @@ int snd_config_search_alias_hooks(snd_config_t *config,
 #define ALSA_CONFIG_PATH_VAR "ALSA_CONFIG_PATH"
 
 /** The name of the default files used by #snd_config_update. */
-#define ALSA_CONFIG_PATH_DEFAULT DATADIR "/alsa/alsa.conf"
+#define ALSA_CONFIG_PATH_DEFAULT ALSA_CONFIG_DIR "/alsa.conf"
 
 /**
  * \ingroup Config
@@ -3059,7 +3081,9 @@ int snd_config_update_r(snd_config_t **_top, snd_config_update_t **_update, cons
 	return 1;
 }
 
+#ifdef HAVE_LIBPTHREAD
 static pthread_mutex_t snd_config_update_mutex = PTHREAD_MUTEX_INITIALIZER;
+#endif
 
 /** 
  * \brief Updates #snd_config by rereading the global configuration files (if needed).
@@ -3078,9 +3102,13 @@ int snd_config_update(void)
 {
 	int err;
 
+#ifdef HAVE_LIBPTHREAD
 	pthread_mutex_lock(&snd_config_update_mutex);
+#endif
 	err = snd_config_update_r(&snd_config, &snd_config_global_update, NULL);
+#ifdef HAVE_LIBPTHREAD
 	pthread_mutex_unlock(&snd_config_update_mutex);
+#endif
 	return err;
 }
 
@@ -3107,15 +3135,18 @@ int snd_config_update_free(snd_config_update_t *update)
  */
 int snd_config_update_free_global(void)
 {
+#ifdef HAVE_LIBPTHREAD
 	pthread_mutex_lock(&snd_config_update_mutex);
+#endif
 	if (snd_config)
 		snd_config_delete(snd_config);
 	snd_config = NULL;
 	if (snd_config_global_update)
 		snd_config_update_free(snd_config_global_update);
 	snd_config_global_update = NULL;
+#ifdef HAVE_LIBPTHREAD
 	pthread_mutex_unlock(&snd_config_update_mutex);
-
+#endif
 	/* FIXME: better to place this in another place... */
 	snd_dlobj_cache_cleanup();
 
@@ -3914,6 +3945,10 @@ int snd_config_expand(snd_config_t *config, snd_config_t *root, const char *args
 	snd_config_t *defs, *subs = NULL, *res;
 	err = snd_config_search(config, "@args", &defs);
 	if (err < 0) {
+		if (args != NULL) {
+			SNDERR("Unknown parameters %s", args);
+			return -EINVAL;
+		}
 		err = snd_config_copy(&res, config);
 		if (err < 0)
 			return err;
@@ -3963,7 +3998,7 @@ int snd_config_expand(snd_config_t *config, snd_config_t *root, const char *args
  * \param name Key suffix.
  * \param result The function puts the handle to the expanded found node at
  *               the address specified by \p result.
- * \return Zero if successful, otherwise a negative error code.
+ * \return A non-negative value if successful, otherwise a negative error code.
  *
  * First the key is tried, then, if nothing is found, base.key is tried.
  * If the value found is a string, this is recursively tried in the
