@@ -35,8 +35,8 @@
 #include "control_local.h"
 
 #ifndef DOC_HIDDEN
-#define SND_FILE_CONTROL	"/dev/snd/controlC%i"
-#define SND_FILE_LOAD		"/dev/aloadC%i"
+#define SND_FILE_CONTROL	ALSA_DEVICE_DIRECTORY "controlC%i"
+#define SND_FILE_LOAD		ALOAD_DEVICE_DIRECTORY "aloadC%i"
 #endif
 
 /**
@@ -47,23 +47,23 @@
 int snd_card_load(int card)
 {
 	int open_dev;
-	char control[32];
+	char control[sizeof(SND_FILE_CONTROL) + 10];
 
 	sprintf(control, SND_FILE_CONTROL, card);
 
 	open_dev = snd_open_device(control, O_RDONLY);
 #ifdef SUPPORT_ALOAD
 	if (open_dev < 0) {
-		char aload[32];
+		char aload[sizeof(SND_FILE_LOAD) + 10];
 		sprintf(aload, SND_FILE_LOAD, card);
 		open_dev = snd_open_device(aload, O_RDONLY);
 	}
 #endif
 	if (open_dev >= 0) {
 		close (open_dev);
-		return 0;
+		return 1;
 	}
-	return open_dev ? 1 : 0;
+	return 0;
 }
 
 /**
@@ -85,7 +85,7 @@ int snd_card_next(int *rcard)
 	card = *rcard;
 	card = card < 0 ? 0 : card + 1;
 	for (; card < 32; card++) {
-		if (!snd_card_load(card)) {
+		if (snd_card_load(card)) {
 			*rcard = card;
 			return 0;
 		}
@@ -112,15 +112,16 @@ int snd_card_get_index(const char *string)
 		return -EINVAL;
 	if ((isdigit(*string) && *(string + 1) == 0) ||
 	    (isdigit(*string) && isdigit(*(string + 1)) && *(string + 2) == 0)) {
-		sscanf(string, "%i", &card);
+		if (sscanf(string, "%i", &card) != 1)
+			return -EINVAL;
 		if (card < 0 || card > 31)
 			return -EINVAL;
-	        if (snd_card_load(card) >= 0)
+		if (snd_card_load(card))
 			return card;
-		return -EINVAL;
+		return -ENODEV;
 	}
 	for (card = 0; card < 32; card++) {
-		if (snd_card_load(card) < 0)
+		if (! snd_card_load(card))
 			continue;
 		if (snd_ctl_hw_open(&handle, NULL, card, 0) < 0)
 			continue;
@@ -129,7 +130,7 @@ int snd_card_get_index(const char *string)
 			continue;
 		}
 		snd_ctl_close(handle);
-		if (!strcmp(info.id, string))
+		if (!strcmp((const char *)info.id, string))
 			return card;
 	}
 	return -ENODEV;
@@ -156,7 +157,7 @@ int snd_card_get_name(int card, char **name)
 		return err;
 	}
 	snd_ctl_close(handle);
-	*name = strdup(info.name);
+	*name = strdup((const char *)info.name);
 	if (*name == NULL)
 		return -ENOMEM;
 	return 0;
@@ -183,7 +184,7 @@ int snd_card_get_longname(int card, char **name)
 		return err;
 	}
 	snd_ctl_close(handle);
-	*name = strdup(info.longname);
+	*name = strdup((const char *)info.longname);
 	if (*name == NULL)
 		return -ENOMEM;
 	return 0;
