@@ -33,7 +33,6 @@
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
-#include <dlfcn.h>
 #include <sys/ioctl.h>
 #include "hwdep_local.h"
 
@@ -207,6 +206,8 @@ int snd_hwdep_close(snd_hwdep_t *hwdep)
 	int err;
   	assert(hwdep);
 	err = hwdep->ops->close(hwdep);
+	if (hwdep->dl_handle)
+		snd_dlclose(hwdep->dl_handle);
 	free(hwdep->name);
 	free(hwdep);
 	return err;
