@@ -31,19 +31,6 @@
 #include <sys/stat.h>
 #include <stdio.h>
 #include "local.h"
-/* linux 2.6.9 workaround */
-#include <linux/version.h>
-#if LINUX_VERSION_CODE == KERNEL_VERSION(2,6,9)
-#include <asm/types.h>
-#ifndef DOC_HIDDEN
-#define __le64 __u64
-#define __le32 __u32
-#define __le16 __u16
-#define __be64 __u64
-#define __be32 __u32
-#define __be16 __u16
-#endif
-#endif /* linux 2.6.9 workaround */
 #include <sound/ainstr_iw.h>
 
 /*
