@@ -47,13 +47,15 @@
 
 typedef enum _selem_ctl_type {
 	CTL_SINGLE,
-	CTL_ENUMLIST,
+	CTL_GLOBAL_ENUM,
 	CTL_GLOBAL_SWITCH,
 	CTL_GLOBAL_VOLUME,
 	CTL_GLOBAL_ROUTE,
+	CTL_PLAYBACK_ENUM,
 	CTL_PLAYBACK_SWITCH,
 	CTL_PLAYBACK_VOLUME,
 	CTL_PLAYBACK_ROUTE,
+	CTL_CAPTURE_ENUM,
 	CTL_CAPTURE_SWITCH,
 	CTL_CAPTURE_VOLUME,
 	CTL_CAPTURE_ROUTE,
@@ -286,7 +288,16 @@ static int elem_read_enum(selem_none_t *s)
 	snd_ctl_elem_value_t *ctl;
 	unsigned int idx;
 	int err;
-	selem_ctl_t *c = &s->ctls[CTL_ENUMLIST];
+	int type;
+	selem_ctl_t *c;
+	type = CTL_GLOBAL_ENUM;
+	if ( (s->selem.caps & (SM_CAP_CENUM | SM_CAP_PENUM)) == (SM_CAP_CENUM | SM_CAP_PENUM) )
+		type = CTL_GLOBAL_ENUM;
+	else if (s->selem.caps & SM_CAP_PENUM)
+		type = CTL_PLAYBACK_ENUM;
+	else if (s->selem.caps & SM_CAP_CENUM)
+		type = CTL_CAPTURE_ENUM;
+	c = &s->ctls[type];
 	snd_ctl_elem_value_alloca(&ctl);
 	if ((err = snd_hctl_elem_read(c->elem, ctl)) < 0)
 		return err;
@@ -319,13 +330,28 @@ static int selem_read(snd_mixer_elem_t *elem)
 	csw = s->str[SM_CAPT].sw;
 	s->str[SM_CAPT].sw = ~0U;
 
-	if (s->ctls[CTL_ENUMLIST].elem) {
+	if (s->ctls[CTL_GLOBAL_ENUM].elem) {
 		err = elem_read_enum(s);
 		if (err < 0)
 			return err;
 		goto __skip_cswitch;
 	}
 
+	if (s->ctls[CTL_CAPTURE_ENUM].elem) {
+		err = elem_read_enum(s);
+		if (err < 0)
+			return err;
+		goto __skip_cswitch;
+	}
+
+	if (s->ctls[CTL_PLAYBACK_ENUM].elem) {
+		err = elem_read_enum(s);
+		if (err < 0)
+			return err;
+		goto __skip_cswitch;
+	}
+
+
 	if (s->ctls[CTL_PLAYBACK_VOLUME].elem)
 		err = elem_read_volume(s, SM_PLAY, CTL_PLAYBACK_VOLUME);
 	else if (s->ctls[CTL_GLOBAL_VOLUME].elem)
@@ -504,7 +530,16 @@ static int elem_write_enum(selem_none_t *s)
 	snd_ctl_elem_value_t *ctl;
 	unsigned int idx;
 	int err;
-	selem_ctl_t *c = &s->ctls[CTL_ENUMLIST];
+	int type;
+	selem_ctl_t *c;
+	type = CTL_GLOBAL_ENUM;
+	if ( (s->selem.caps & (SM_CAP_CENUM | SM_CAP_PENUM) ) == (SM_CAP_CENUM | SM_CAP_PENUM) )
+		type = CTL_GLOBAL_ENUM;
+	else if (s->selem.caps & SM_CAP_PENUM)
+		type = CTL_PLAYBACK_ENUM;
+	else if (s->selem.caps & SM_CAP_CENUM)
+		type = CTL_CAPTURE_ENUM;
+	c = &s->ctls[type];
 	snd_ctl_elem_value_alloca(&ctl);
 	if ((err = snd_hctl_elem_read(c->elem, ctl)) < 0)
 		return err;
@@ -515,7 +550,7 @@ static int elem_write_enum(selem_none_t *s)
 	return 0;
 }
 
-static int selem_write(snd_mixer_elem_t *elem)
+static int selem_write_main(snd_mixer_elem_t *elem)
 {
 	selem_none_t *s;
 	unsigned int idx;
@@ -524,7 +559,13 @@ static int selem_write(snd_mixer_elem_t *elem)
 	assert(snd_mixer_elem_get_type(elem) == SND_MIXER_ELEM_SIMPLE);
 	s = snd_mixer_elem_get_private(elem);
 
-	if (s->ctls[CTL_ENUMLIST].elem)
+	if (s->ctls[CTL_GLOBAL_ENUM].elem)
+		return elem_write_enum(s);
+
+	if (s->ctls[CTL_PLAYBACK_ENUM].elem)
+		return elem_write_enum(s);
+
+	if (s->ctls[CTL_CAPTURE_ENUM].elem)
 		return elem_write_enum(s);
 
 	if (s->ctls[CTL_SINGLE].elem) {
@@ -598,6 +639,16 @@ static int selem_write(snd_mixer_elem_t *elem)
 	return 0;
 }
 
+static int selem_write(snd_mixer_elem_t *elem)
+{
+	int err;
+	
+	err = selem_write_main(elem);
+	if (err < 0)
+		selem_read(elem);
+	return err;
+}
+
 static void selem_free(snd_mixer_elem_t *elem)
 {
 	selem_none_t *simple = snd_mixer_elem_get_private(elem);
@@ -616,6 +667,7 @@ static int simple_update(snd_mixer_elem_t *melem)
 	unsigned int caps, pchannels, cchannels;
 	long pmin, pmax, cmin, cmax;
 	selem_ctl_t *ctl;
+	const char *name;
 
 	caps = 0;
 	pchannels = 0;
@@ -624,6 +676,7 @@ static int simple_update(snd_mixer_elem_t *melem)
 	cmin = cmax = 0;
 	assert(snd_mixer_elem_get_type(melem) == SND_MIXER_ELEM_SIMPLE);
 	simple = snd_mixer_elem_get_private(melem);
+	name = snd_mixer_selem_get_name(melem);
 	ctl = &simple->ctls[CTL_SINGLE];
 	if (ctl->elem) {
 		pchannels = cchannels = ctl->values;
@@ -723,12 +776,23 @@ static int simple_update(snd_mixer_elem_t *melem)
 		caps |= SM_CAP_CSWITCH | SM_CAP_CSWITCH_EXCL;
 		caps &= ~SM_CAP_GSWITCH;
 	}
-	ctl = &simple->ctls[CTL_ENUMLIST];
+	ctl = &simple->ctls[CTL_GLOBAL_ENUM];
 	if (ctl->elem) {
 		if (pchannels < ctl->values)
 			pchannels = ctl->values;
-		/* FIXME: differentiate some controls */
-		caps |= SM_CAP_PENUM|SM_CAP_CENUM;
+		caps |= SM_CAP_PENUM | SM_CAP_CENUM;
+	}
+	ctl = &simple->ctls[CTL_PLAYBACK_ENUM];
+	if (ctl->elem) {
+		if (pchannels < ctl->values)
+			pchannels = ctl->values;
+		caps |= SM_CAP_PENUM;
+	}
+	ctl = &simple->ctls[CTL_CAPTURE_ENUM];
+	if (ctl->elem) {
+		if (pchannels < ctl->values)
+			pchannels = ctl->values;
+		caps |= SM_CAP_CENUM;
 	}
 	if (pchannels > 32)
 		pchannels = 32;
@@ -819,12 +883,15 @@ static struct suf {
 	const char *suffix;
 	selem_ctl_type_t type;
 } suffixes[] = {
+	{" Playback Enum", CTL_PLAYBACK_ENUM},
 	{" Playback Switch", CTL_PLAYBACK_SWITCH},
 	{" Playback Route", CTL_PLAYBACK_ROUTE},
 	{" Playback Volume", CTL_PLAYBACK_VOLUME},
+	{" Capture Enum", CTL_CAPTURE_ENUM},
 	{" Capture Switch", CTL_CAPTURE_SWITCH},
 	{" Capture Route", CTL_CAPTURE_ROUTE},
 	{" Capture Volume", CTL_CAPTURE_VOLUME},
+	{" Enum", CTL_GLOBAL_ENUM},
 	{" Switch", CTL_GLOBAL_SWITCH},
 	{" Route", CTL_GLOBAL_ROUTE},
 	{" Volume", CTL_GLOBAL_VOLUME},
@@ -920,16 +987,33 @@ static int is_ops(snd_mixer_elem_t *elem, int dir, int cmd, int val)
 
 	case SM_OPS_IS_ENUMERATED:
 		if (val == 1) {
-			if (dir == SM_PLAY && (s->selem.caps & SM_CAP_PENUM))
+			if (dir == SM_PLAY && (s->selem.caps & SM_CAP_PENUM) && !(s->selem.caps & SM_CAP_CENUM) )
 				return 1;
-			return !!(s->selem.caps & SM_CAP_CENUM);
+			if (dir == SM_CAPT && (s->selem.caps & SM_CAP_CENUM) && !(s->selem.caps & SM_CAP_PENUM) )
+				return 1;
+			return 0;
 		}
-		return s->ctls[CTL_ENUMLIST].elem != 0;
+		if (s->selem.caps & (SM_CAP_CENUM | SM_CAP_PENUM) )
+			return 1;
+		return 0;
 	
 	case SM_OPS_IS_ENUMCNT:
-		if (! s->ctls[CTL_ENUMLIST].elem)
-			return -EINVAL;
-		return s->ctls[CTL_ENUMLIST].max;
+		/* Both */
+		if ( (s->selem.caps & (SM_CAP_CENUM | SM_CAP_PENUM)) == (SM_CAP_CENUM | SM_CAP_PENUM) ) {
+			if (! s->ctls[CTL_GLOBAL_ENUM].elem)
+				return -EINVAL;
+			return s->ctls[CTL_GLOBAL_ENUM].max;
+		/* Only Playback */
+		} else if (s->selem.caps & SM_CAP_PENUM ) {
+			if (! s->ctls[CTL_PLAYBACK_ENUM].elem)
+				return -EINVAL;
+			return s->ctls[CTL_PLAYBACK_ENUM].max;
+		/* Only Capture */
+		} else if (s->selem.caps & SM_CAP_CENUM ) {
+			if (! s->ctls[CTL_CAPTURE_ENUM].elem)
+				return -EINVAL;
+			return s->ctls[CTL_CAPTURE_ENUM].max;
+		}
 
 	}
 	
@@ -945,14 +1029,6 @@ static int get_range_ops(snd_mixer_elem_t *elem, int dir,
 	return 0;
 }
 
-static int get_dB_range_ops(snd_mixer_elem_t *elem ATTRIBUTE_UNUSED,
-			    int dir ATTRIBUTE_UNUSED,
-			    long *min ATTRIBUTE_UNUSED,
-			    long *max ATTRIBUTE_UNUSED)
-{
-	return -ENXIO;
-}
-
 static int set_range_ops(snd_mixer_elem_t *elem, int dir,
 			 long min, long max)
 {
@@ -977,8 +1053,17 @@ static int get_volume_ops(snd_mixer_elem_t *elem, int dir,
 	return 0;
 }
 
+static int init_db_range(snd_hctl_elem_t *ctl, struct selem_str *rec);
+
 /* convert to index of integer array */
+#ifndef DOC_HIDDEN
 #define int_index(size)	(((size) + sizeof(int) - 1) / sizeof(int))
+#endif
+
+/* max size of a TLV entry for dB information (including compound one) */
+#ifndef DOC_HIDDEN
+#define MAX_TLV_RANGE_SIZE	256
+#endif
 
 /* parse TLV stream and retrieve dB information
  * return 0 if successly found and stored to rec,
@@ -1014,11 +1099,29 @@ static int parse_db_range(struct selem_str *rec, unsigned int *tlv,
 		}
 		break;
 	case SND_CTL_TLVT_DB_SCALE:
+#ifndef HAVE_SOFT_FLOAT
+	case SND_CTL_TLVT_DB_LINEAR:
+#endif
+	case SND_CTL_TLVT_DB_RANGE: {
+		unsigned int minsize;
+		if (type == SND_CTL_TLVT_DB_RANGE)
+			minsize = 4 * sizeof(int);
+		else
+			minsize = 2 * sizeof(int);
+		if (size < minsize) {
+			SNDERR("Invalid dB_scale TLV size");
+			return -EINVAL;
+		}
+		if (size > MAX_TLV_RANGE_SIZE) {
+			SNDERR("Too big dB_scale TLV size: %d", size);
+			return -EINVAL;
+		}
 		rec->db_info = malloc(size + sizeof(int) * 2);
 		if (! rec->db_info)
 			return -ENOMEM;
 		memcpy(rec->db_info, tlv, size + sizeof(int) * 2);
 		return 0;
+	}
 	default:
 		break;
 	}
@@ -1027,24 +1130,77 @@ static int parse_db_range(struct selem_str *rec, unsigned int *tlv,
 
 /* convert the given raw volume value to a dB gain
  */
-static int convert_db_range(struct selem_str *rec, long volume, long *db_gain)
-{
-	int min, step, mute;
 
-	switch (rec->db_info[0]) {
-	case SND_CTL_TLVT_DB_SCALE:
-		min = rec->db_info[2];
-		step = (rec->db_info[3] & 0xffff);
-		mute = (rec->db_info[3] >> 16) & 1;
-		if (mute && volume == rec->min)
-			*db_gain = -9999999;
+static int do_convert_to_dB(unsigned int *tlv, long rangemin, long rangemax,
+			    long volume, long *db_gain)
+{
+	switch (tlv[0]) {
+	case SND_CTL_TLVT_DB_RANGE: {
+		unsigned int pos, len;
+		len = int_index(tlv[1]);
+		if (len > MAX_TLV_RANGE_SIZE)
+			return -EINVAL;
+		pos = 2;
+		while (pos + 4 <= len) {
+			rangemin = (int)tlv[pos];
+			rangemax = (int)tlv[pos + 1];
+			if (volume >= rangemin && volume <= rangemax)
+				return do_convert_to_dB(tlv + pos + 2,
+							rangemin, rangemax,
+							volume, db_gain);
+			pos += int_index(tlv[pos + 3]) + 4;
+		}
+		return -EINVAL;
+	}
+	case SND_CTL_TLVT_DB_SCALE: {
+		int min, step, mute;
+		min = tlv[2];
+		step = (tlv[3] & 0xffff);
+		mute = (tlv[3] >> 16) & 1;
+		if (mute && volume == rangemin)
+			*db_gain = SND_CTL_TLV_DB_GAIN_MUTE;
 		else
-			*db_gain = (volume - rec->min) * step + min;
+			*db_gain = (volume - rangemin) * step + min;
 		return 0;
 	}
+#ifndef HAVE_SOFT_FLOAT
+	case SND_CTL_TLVT_DB_LINEAR: {
+		int mindb = tlv[2];
+		int maxdb = tlv[3];
+		if (volume <= rangemin || rangemax <= rangemin)
+			*db_gain = mindb;
+		else if (volume >= rangemax)
+			*db_gain = maxdb;
+		else {
+			double val = (double)(volume - rangemin) /
+				(double)(rangemax - rangemin);
+			if (mindb <= SND_CTL_TLV_DB_GAIN_MUTE)
+				*db_gain = (long)(100.0 * 20.0 * log10(val)) +
+					maxdb;
+			else {
+				/* FIXME: precalculate and cache these values */
+				double lmin = pow(10.0, mindb/2000.0);
+				double lmax = pow(10.0, maxdb/2000.0);
+				val = (lmax - lmin) * val + lmin;
+				*db_gain = (long)(100.0 * 20.0 * log10(val));
+			}
+		}
+		return 0;
+	}
+#endif
+	}
 	return -EINVAL;
 }
 
+static int convert_to_dB(snd_hctl_elem_t *ctl, struct selem_str *rec,
+			 long volume, long *db_gain)
+{
+	if (init_db_range(ctl, rec) < 0)
+		return -EINVAL;
+	return do_convert_to_dB(rec->db_info, rec->min, rec->max,
+				volume, db_gain);
+}
+
 /* initialize dB range information, reading TLV via hcontrol
  */
 static int init_db_range(snd_hctl_elem_t *ctl, struct selem_str *rec)
@@ -1053,6 +1209,11 @@ static int init_db_range(snd_hctl_elem_t *ctl, struct selem_str *rec)
 	unsigned int *tlv = NULL;
 	const unsigned int tlv_size = 4096;
 
+	if (rec->db_init_error)
+		return -EINVAL;
+	if (rec->db_initialized)
+		return 0;
+
 	snd_ctl_elem_info_alloca(&info);
 	if (snd_hctl_elem_info(ctl, info) < 0)
 		goto error;
@@ -1075,18 +1236,178 @@ static int init_db_range(snd_hctl_elem_t *ctl, struct selem_str *rec)
 	return -EINVAL;
 }
 
-static int get_dB_gain(snd_hctl_elem_t *ctl, struct selem_str *rec,
-		       long volume, long *db_gain)
+/* get selem_ctl for TLV access */
+static selem_ctl_t *get_selem_ctl(selem_none_t *s, int dir)
 {
-	if (rec->db_init_error)
-		return -EINVAL;
-	if (! rec->db_initialized) {
-		if (init_db_range(ctl, rec) < 0)
-			return -EINVAL;
+	selem_ctl_t *c;
+	if (dir == SM_PLAY)
+		c = &s->ctls[CTL_PLAYBACK_VOLUME];
+	else if (dir == SM_CAPT)
+		c = &s->ctls[CTL_CAPTURE_VOLUME];
+	else
+		return NULL;
+	if (! c->elem) {
+		c = &s->ctls[CTL_GLOBAL_VOLUME];
+		if (! c->elem)
+			return NULL;
 	}
-	return convert_db_range(rec, volume, db_gain);
+	if (c->type != SND_CTL_ELEM_TYPE_INTEGER)
+		return NULL;
+	return c;
+}
+
+/* Get the dB min/max values
+ */
+static int do_get_dB_range(unsigned int *tlv, long rangemin, long rangemax,
+			   long *min, long *max)
+{
+	switch (tlv[0]) {
+	case SND_CTL_TLVT_DB_RANGE: {
+		unsigned int pos, len;
+		len = int_index(tlv[1]);
+		if (len > MAX_TLV_RANGE_SIZE)
+			return -EINVAL;
+		pos = 2;
+		while (pos + 4 <= len) {
+			long rmin, rmax;
+			rangemin = (int)tlv[pos];
+			rangemax = (int)tlv[pos + 1];
+			do_get_dB_range(tlv + pos + 2, rangemin, rangemax,
+					&rmin, &rmax);
+			if (pos > 2) {
+				if (rmin < *min)
+					*min = rmin;
+				if (rmax > *max)
+					*max = rmax;
+			} else {
+				*min = rmin;
+				*max = rmax;
+			}
+			pos += int_index(tlv[pos + 3]) + 4;
+		}
+		return 0;
+	}
+	case SND_CTL_TLVT_DB_SCALE: {
+		int step;
+		*min = (int)tlv[2];
+		step = (tlv[3] & 0xffff);
+		*max = *min + (long)(step * (rangemax - rangemin));
+		return 0;
+	}
+	case SND_CTL_TLVT_DB_LINEAR:
+		*min = (int)tlv[2];
+		*max = (int)tlv[3];
+		return 0;
+	}
+	return -EINVAL;
+}
+
+static int get_dB_range(snd_hctl_elem_t *ctl, struct selem_str *rec,
+			long *min, long *max)
+{
+	if (init_db_range(ctl, rec) < 0)
+		return -EINVAL;
+
+	return do_get_dB_range(rec->db_info, rec->min, rec->max, min, max);
 }
 	
+static int get_dB_range_ops(snd_mixer_elem_t *elem, int dir,
+			    long *min, long *max)
+{
+	selem_none_t *s = snd_mixer_elem_get_private(elem);
+	selem_ctl_t *c;
+
+	c = get_selem_ctl(s, dir);
+	if (! c)
+		return -EINVAL;
+	return get_dB_range(c->elem, &s->str[dir], min, max);
+}
+
+/* Convert from dB gain to the corresponding raw value.
+ * The value is round up when xdir > 0.
+ */
+static int do_convert_from_dB(unsigned int *tlv, long rangemin, long rangemax,
+			      long db_gain, long *value, int xdir)
+{
+	switch (tlv[0]) {
+	case SND_CTL_TLVT_DB_RANGE: {
+		unsigned int pos, len;
+		len = int_index(tlv[1]);
+		if (len > MAX_TLV_RANGE_SIZE)
+			return -EINVAL;
+		pos = 2;
+		while (pos + 4 <= len) {
+			long dbmin, dbmax;
+			rangemin = (int)tlv[pos];
+			rangemax = (int)tlv[pos + 1];
+			if (! do_get_dB_range(tlv + pos + 2, rangemin, rangemax,
+					      &dbmin, &dbmax) &&
+			    db_gain >= dbmin && db_gain <= dbmax)
+				return do_convert_from_dB(tlv + pos + 2,
+							  rangemin, rangemax,
+							  db_gain, value, xdir);
+			pos += int_index(tlv[pos + 3]) + 4;
+		}
+		return -EINVAL;
+	}
+	case SND_CTL_TLVT_DB_SCALE: {
+		int min, step, max;
+		min = tlv[2];
+		step = (tlv[3] & 0xffff);
+		max = min + (int)(step * (rangemax - rangemin));
+		if (db_gain <= min)
+			*value = rangemin;
+		else if (db_gain >= max)
+			*value = rangemax;
+		else {
+			long v = (db_gain - min) * (rangemax - rangemin);
+			if (xdir > 0)
+				v += (max - min) - 1;
+			v = v / (max - min) + rangemin;
+			*value = v;
+		}
+		return 0;
+	}
+#ifndef HAVE_SOFT_FLOAT
+	case SND_CTL_TLVT_DB_LINEAR: {
+		int min, max;
+		min = tlv[2];
+		max = tlv[3];
+		if (db_gain <= min)
+			*value = rangemin;
+		else if (db_gain >= max)
+			*value = rangemax;
+		else {
+			/* FIXME: precalculate and cache vmin and vmax */
+			double vmin, vmax, v;
+			vmin = (min <= SND_CTL_TLV_DB_GAIN_MUTE) ? 0.0 :
+				pow(10.0,  (double)min / 2000.0);
+			vmax = !max ? 1.0 : pow(10.0,  (double)max / 2000.0);
+			v = pow(10.0, (double)db_gain / 2000.0);
+			v = (v - vmin) * (rangemax - rangemin) / (vmax - vmin);
+			if (xdir > 0)
+				v = ceil(v);
+			*value = (long)v + rangemin;
+		}
+		return 0;
+	}
+#endif
+	default:
+		break;
+	}
+	return -EINVAL;
+}
+
+static int convert_from_dB(snd_hctl_elem_t *ctl, struct selem_str *rec,
+			   long db_gain, long *value, int xdir)
+{
+	if (init_db_range(ctl, rec) < 0)
+		return -EINVAL;
+
+	return do_convert_from_dB(rec->db_info, rec->min, rec->max,
+				  db_gain, value, xdir);
+}
+
 static int get_dB_ops(snd_mixer_elem_t *elem,
                       int dir,
                       snd_mixer_selem_channel_id_t channel,
@@ -1094,25 +1415,15 @@ static int get_dB_ops(snd_mixer_elem_t *elem,
 {
 	selem_none_t *s = snd_mixer_elem_get_private(elem);
 	selem_ctl_t *c;
-	int err = -EINVAL;
+	int err;
 	long volume, db_gain;
 
-	if (dir == SM_PLAY)
-		c = &s->ctls[CTL_PLAYBACK_VOLUME];
-	else if (dir == SM_CAPT)
-		c = &s->ctls[CTL_CAPTURE_VOLUME];
-	else
-		goto _err;
-	if (! c->elem) {
-		c = &s->ctls[CTL_GLOBAL_VOLUME];
-		if (! c->elem)
-			goto _err;
-	}
-	if (c->type != SND_CTL_ELEM_TYPE_INTEGER)
-		goto _err;
+	c = get_selem_ctl(s, dir);
+	if (! c)
+		return -EINVAL;
 	if ((err = get_volume_ops(elem, dir, channel, &volume)) < 0)
 		goto _err;
-	if ((err = get_dB_gain(c->elem, &s->str[dir], volume, &db_gain)) < 0)
+	if ((err = convert_to_dB(c->elem, &s->str[dir], volume, &db_gain)) < 0)
 		goto _err;
 	err = 0;
 	*value = db_gain;
@@ -1142,13 +1453,22 @@ static int set_volume_ops(snd_mixer_elem_t *elem, int dir,
 	return 0;
 }
 
-static int set_dB_ops(snd_mixer_elem_t *elem ATTRIBUTE_UNUSED,
-		      int dir ATTRIBUTE_UNUSED,
-		      snd_mixer_selem_channel_id_t channel ATTRIBUTE_UNUSED,
-		      long value ATTRIBUTE_UNUSED,
-		      int xdir ATTRIBUTE_UNUSED)
+static int set_dB_ops(snd_mixer_elem_t *elem, int dir,
+		      snd_mixer_selem_channel_id_t channel,
+		      long db_gain, int xdir)
 {
-	return -ENXIO;
+	selem_none_t *s = snd_mixer_elem_get_private(elem);
+	selem_ctl_t *c;
+	long value;
+	int err;
+
+	c = get_selem_ctl(s, dir);
+	if (! c)
+		return -EINVAL;
+	err = convert_from_dB(c->elem, &s->str[dir], db_gain, &value, xdir);
+	if (err < 0)
+		return err;
+	return set_volume_ops(elem, dir, channel, value);
 }
 
 static int set_switch_ops(snd_mixer_elem_t *elem, int dir,
@@ -1178,10 +1498,20 @@ static int enum_item_name_ops(snd_mixer_elem_t *elem,
 	selem_none_t *s = snd_mixer_elem_get_private(elem);
 	snd_ctl_elem_info_t *info;
 	snd_hctl_elem_t *helem;
+	int type;
 
-	helem = s->ctls[CTL_ENUMLIST].elem;
+	type = CTL_GLOBAL_ENUM;
+	helem = s->ctls[type].elem;
+	if (!helem) {
+		type = CTL_PLAYBACK_ENUM;
+		helem = s->ctls[type].elem;
+	}
+	if (!helem) {
+		type = CTL_CAPTURE_ENUM;
+		helem = s->ctls[type].elem;
+	}
 	assert(helem);
-	if (item >= (unsigned int)s->ctls[CTL_ENUMLIST].max)
+	if (item >= (unsigned int)s->ctls[type].max)
 		return -EINVAL;
 	snd_ctl_elem_info_alloca(&info);
 	snd_hctl_elem_info(helem, info);
@@ -1202,7 +1532,9 @@ static int get_enum_item_ops(snd_mixer_elem_t *elem,
 
 	if ((unsigned int) channel >= s->str[0].channels)
 		return -EINVAL;
-	helem = s->ctls[CTL_ENUMLIST].elem;
+	helem = s->ctls[CTL_GLOBAL_ENUM].elem;
+	if (!helem) helem = s->ctls[CTL_PLAYBACK_ENUM].elem;
+	if (!helem) helem = s->ctls[CTL_CAPTURE_ENUM].elem;
 	assert(helem);
 	snd_ctl_elem_value_alloca(&ctl);
 	err = snd_hctl_elem_read(helem, ctl);
@@ -1219,17 +1551,30 @@ static int set_enum_item_ops(snd_mixer_elem_t *elem,
 	snd_ctl_elem_value_t *ctl;
 	snd_hctl_elem_t *helem;
 	int err;
+	int type;
 
-	if ((unsigned int) channel >= s->str[0].channels)
+	if ((unsigned int) channel >= s->str[0].channels) {
 		return -EINVAL;
-	helem = s->ctls[CTL_ENUMLIST].elem;
+	}
+	type = CTL_GLOBAL_ENUM;
+	helem = s->ctls[type].elem;
+	if (!helem) {
+		type = CTL_PLAYBACK_ENUM;
+		helem = s->ctls[type].elem;
+	}
+	if (!helem) {
+		type = CTL_CAPTURE_ENUM;
+		helem = s->ctls[type].elem;
+	}
 	assert(helem);
-	if (item >= (unsigned int)s->ctls[CTL_ENUMLIST].max)
+	if (item >= (unsigned int)s->ctls[type].max) {
 		return -EINVAL;
+	}
 	snd_ctl_elem_value_alloca(&ctl);
 	err = snd_hctl_elem_read(helem, ctl);
-	if (err < 0)
+	if (err < 0) {
 		return err;
+	}
 	snd_ctl_elem_value_set_enumerated(ctl, channel, item);
 	return snd_hctl_elem_write(helem, ctl);
 }
@@ -1273,7 +1618,7 @@ static int simple_add1(snd_mixer_class_t *class, const char *name,
 	switch (type) {
 	case CTL_SINGLE:
 		if (ctype == SND_CTL_ELEM_TYPE_ENUMERATED)
-			type = CTL_ENUMLIST;
+			type = CTL_GLOBAL_ENUM;
 		else if (ctype != SND_CTL_ELEM_TYPE_BOOLEAN &&
 		         ctype != SND_CTL_ELEM_TYPE_INTEGER)
 			return 0;
@@ -1284,7 +1629,7 @@ static int simple_add1(snd_mixer_class_t *class, const char *name,
 	{
 		unsigned int n;
 		if (ctype == SND_CTL_ELEM_TYPE_ENUMERATED) {
-			type = CTL_ENUMLIST;
+			type = CTL_GLOBAL_ENUM;
 			break;
 		}
 		if (ctype != SND_CTL_ELEM_TYPE_BOOLEAN)
@@ -1299,7 +1644,7 @@ static int simple_add1(snd_mixer_class_t *class, const char *name,
 	case CTL_PLAYBACK_SWITCH:
 	case CTL_CAPTURE_SWITCH:
 		if (ctype == SND_CTL_ELEM_TYPE_ENUMERATED) {
-			type = CTL_ENUMLIST;
+			type = CTL_GLOBAL_ENUM;
 			break;
 		}
 		if (ctype != SND_CTL_ELEM_TYPE_BOOLEAN)
@@ -1309,7 +1654,7 @@ static int simple_add1(snd_mixer_class_t *class, const char *name,
 	case CTL_PLAYBACK_VOLUME:
 	case CTL_CAPTURE_VOLUME:
 		if (ctype == SND_CTL_ELEM_TYPE_ENUMERATED) {
-			type = CTL_ENUMLIST;
+			type = CTL_GLOBAL_ENUM;
 			break;
 		}
 		if (ctype != SND_CTL_ELEM_TYPE_INTEGER)
@@ -1319,6 +1664,12 @@ static int simple_add1(snd_mixer_class_t *class, const char *name,
 		if (ctype != SND_CTL_ELEM_TYPE_ENUMERATED)
 			return 0;
 		break;
+	case CTL_GLOBAL_ENUM:
+	case CTL_PLAYBACK_ENUM:
+	case CTL_CAPTURE_ENUM:
+		if (ctype != SND_CTL_ELEM_TYPE_ENUMERATED)
+			return 0;
+		break;
 	default:
 		assert(0);
 		break;
@@ -1351,7 +1702,7 @@ static int simple_add1(snd_mixer_class_t *class, const char *name,
 		snd_mixer_selem_id_free(id);
 	}
 	if (simple->ctls[type].elem) {
-		SNDERR("helem (%s,'%s',%li,%li,%li) appears twice or more",
+		SNDERR("helem (%s,'%s',%u,%u,%u) appears twice or more",
 				snd_ctl_elem_iface_name(snd_hctl_elem_get_interface(helem)),
 				snd_hctl_elem_get_name(helem),
 				snd_hctl_elem_get_index(helem),
@@ -1364,7 +1715,9 @@ static int simple_add1(snd_mixer_class_t *class, const char *name,
 	simple->ctls[type].type = snd_ctl_elem_info_get_type(info);
 	simple->ctls[type].inactive = snd_ctl_elem_info_is_inactive(info);
 	simple->ctls[type].values = values;
-	if (type == CTL_ENUMLIST) {
+	if ( (type == CTL_GLOBAL_ENUM) ||
+	     (type == CTL_PLAYBACK_ENUM) ||
+	     (type == CTL_CAPTURE_ENUM) ) {
 		simple->ctls[type].min = 0;
 		simple->ctls[type].max = snd_ctl_elem_info_get_items(info);
 	} else {
