@@ -34,6 +34,7 @@
 #include <string.h>
 #include <fcntl.h>
 #include <ctype.h>
+#include <grp.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <sys/shm.h>
@@ -102,48 +103,52 @@ static void share_areas(snd_pcm_direct_t *dshare,
 static void snd_pcm_dshare_sync_area(snd_pcm_t *pcm)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
-	snd_pcm_uframes_t appl_ptr, slave_appl_ptr, slave_bsize;
-	snd_pcm_uframes_t size, slave_hw_ptr;
+	snd_pcm_uframes_t slave_hw_ptr, slave_appl_ptr, slave_size;
+	snd_pcm_uframes_t appl_ptr, size;
 	const snd_pcm_channel_area_t *src_areas, *dst_areas;
 	
 	/* calculate the size to transfer */
 	size = dshare->appl_ptr - dshare->last_appl_ptr;
 	if (! size)
 		return;
-	slave_bsize = dshare->shmptr->s.buffer_size;
 	slave_hw_ptr = dshare->slave_hw_ptr;
 	/* don't write on the last active period - this area may be cleared
 	 * by the driver during write operation...
 	 */
-	slave_hw_ptr -= slave_hw_ptr % dshare->shmptr->s.period_size;
-	slave_hw_ptr += slave_bsize;
-	if (dshare->slave_hw_ptr > dshare->slave_appl_ptr)
-		slave_hw_ptr -= dshare->shmptr->s.boundary;
-	if (dshare->slave_appl_ptr + size >= slave_hw_ptr)
-		size = slave_hw_ptr - dshare->slave_appl_ptr;
+	slave_hw_ptr -= slave_hw_ptr % dshare->slave_period_size;
+	slave_hw_ptr += dshare->slave_buffer_size;
+	if (dshare->slave_hw_ptr > dshare->slave_boundary)
+		slave_hw_ptr -= dshare->slave_boundary;
+	if (slave_hw_ptr < dshare->slave_appl_ptr)
+		slave_size = slave_hw_ptr + (dshare->slave_boundary - dshare->slave_appl_ptr);
+	else
+		slave_size = slave_hw_ptr - dshare->slave_appl_ptr;
+	if (slave_size < size)
+		size = slave_size;
 	if (! size)
 		return;
+
 	/* add sample areas here */
 	src_areas = snd_pcm_mmap_areas(pcm);
 	dst_areas = snd_pcm_mmap_areas(dshare->spcm);
 	appl_ptr = dshare->last_appl_ptr % pcm->buffer_size;
 	dshare->last_appl_ptr += size;
 	dshare->last_appl_ptr %= pcm->boundary;
-	slave_appl_ptr = dshare->slave_appl_ptr % slave_bsize;
+	slave_appl_ptr = dshare->slave_appl_ptr % dshare->slave_buffer_size;
 	dshare->slave_appl_ptr += size;
-	dshare->slave_appl_ptr %= dshare->shmptr->s.boundary;
+	dshare->slave_appl_ptr %= dshare->slave_boundary;
 	for (;;) {
 		snd_pcm_uframes_t transfer = size;
 		if (appl_ptr + transfer > pcm->buffer_size)
 			transfer = pcm->buffer_size - appl_ptr;
-		if (slave_appl_ptr + transfer > slave_bsize)
-			transfer = slave_bsize - slave_appl_ptr;
+		if (slave_appl_ptr + transfer > dshare->slave_buffer_size)
+			transfer = dshare->slave_buffer_size - slave_appl_ptr;
 		share_areas(dshare, src_areas, dst_areas, appl_ptr, slave_appl_ptr, transfer);
 		size -= transfer;
 		if (! size)
 			break;
 		slave_appl_ptr += transfer;
-		slave_appl_ptr %= slave_bsize;
+		slave_appl_ptr %= dshare->slave_buffer_size;
 		appl_ptr += transfer;
 		appl_ptr %= pcm->buffer_size;
 	}
@@ -161,7 +166,7 @@ static int snd_pcm_dshare_sync_ptr(snd_pcm_t *pcm)
 	switch (snd_pcm_state(dshare->spcm)) {
 	case SND_PCM_STATE_DISCONNECTED:
 		dshare->state = SNDRV_PCM_STATE_DISCONNECTED;
-		return -ENOTTY;
+		return -ENODEV;
 	default:
 		break;
 	}
@@ -177,7 +182,7 @@ static int snd_pcm_dshare_sync_ptr(snd_pcm_t *pcm)
 		/* not really started yet - don't update hw_ptr */
 		return 0;
 	if (diff < 0) {
-		slave_hw_ptr += dshare->shmptr->s.boundary;
+		slave_hw_ptr += dshare->slave_boundary;
 		diff = slave_hw_ptr - old_slave_hw_ptr;
 	}
 	dshare->hw_ptr += diff;
@@ -267,7 +272,7 @@ static int snd_pcm_dshare_delay(snd_pcm_t *pcm, snd_pcm_sframes_t *delayp)
 	case SNDRV_PCM_STATE_XRUN:
 		return -EPIPE;
 	case SNDRV_PCM_STATE_DISCONNECTED:
-		return -ENOTTY;
+		return -ENODEV;
 	default:
 		return -EBADFD;
 	}
@@ -287,7 +292,7 @@ static int snd_pcm_dshare_hwsync(snd_pcm_t *pcm)
 	case SNDRV_PCM_STATE_XRUN:
 		return -EPIPE;
 	case SNDRV_PCM_STATE_DISCONNECTED:
-		return -ENOTTY;
+		return -ENODEV;
 	default:
 		return -EBADFD;
 	}
@@ -298,7 +303,6 @@ static int snd_pcm_dshare_prepare(snd_pcm_t *pcm)
 	snd_pcm_direct_t *dshare = pcm->private_data;
 
 	snd_pcm_direct_check_interleave(dshare, pcm);
-	// assert(pcm->boundary == dshare->shmptr->s.boundary);	/* for sure */
 	dshare->state = SND_PCM_STATE_PREPARED;
 	dshare->appl_ptr = dshare->last_appl_ptr = 0;
 	dshare->hw_ptr = 0;
@@ -381,6 +385,12 @@ static int snd_pcm_dshare_drain(snd_pcm_t *pcm)
 			return 0;
 		}
 	}
+
+	if (dshare->state == SND_PCM_STATE_XRUN) {
+		snd_pcm_dshare_drop(pcm);
+		return 0;
+	}
+
 	stop_threshold = pcm->stop_threshold;
 	if (pcm->stop_threshold > pcm->buffer_size)
 		pcm->stop_threshold = pcm->buffer_size;
@@ -430,13 +440,6 @@ static snd_pcm_sframes_t snd_pcm_dshare_forward(snd_pcm_t *pcm, snd_pcm_uframes_
 	return frames;
 }
 
-static int snd_pcm_dshare_resume(snd_pcm_t *pcm)
-{
-	snd_pcm_direct_t *dshare = pcm->private_data;
-	snd_pcm_resume(dshare->spcm);
-	return 0;
-}
-
 static snd_pcm_sframes_t snd_pcm_dshare_readi(snd_pcm_t *pcm ATTRIBUTE_UNUSED, void *buffer ATTRIBUTE_UNUSED, snd_pcm_uframes_t size ATTRIBUTE_UNUSED)
 {
 	return -ENODEV;
@@ -461,10 +464,11 @@ static int snd_pcm_dshare_close(snd_pcm_t *pcm)
  		snd_pcm_direct_server_discard(dshare);
  	if (dshare->client)
  		snd_pcm_direct_client_discard(dshare);
-	snd_pcm_direct_shm_discard(dshare);
-	snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
-	if (dshare->bindings)
-		free(dshare->bindings);
+	if (snd_pcm_direct_shm_discard(dshare))
+		snd_pcm_direct_semaphore_discard(dshare);
+	else
+		snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
+	free(dshare->bindings);
 	pcm->private_data = NULL;
 	free(dshare);
 	return 0;
@@ -520,9 +524,9 @@ static void snd_pcm_dshare_dump(snd_pcm_t *pcm, snd_output_t *out)
 {
 	snd_pcm_direct_t *dshare = pcm->private_data;
 
-	snd_output_printf(out, "Direct Stream Mixing PCM\n");
+	snd_output_printf(out, "Direct Share PCM\n");
 	if (pcm->setup) {
-		snd_output_printf(out, "\nIts setup is:\n");
+		snd_output_printf(out, "Its setup is:\n");
 		snd_pcm_dump_setup(pcm, out);
 	}
 	if (dshare->spcm)
@@ -557,9 +561,9 @@ static snd_pcm_fast_ops_t snd_pcm_dshare_fast_ops = {
 	.pause = snd_pcm_dshare_pause,
 	.rewind = snd_pcm_dshare_rewind,
 	.forward = snd_pcm_dshare_forward,
-	.resume = snd_pcm_dshare_resume,
-	.link_fd = NULL,
+	.resume = snd_pcm_direct_resume,
 	.link = NULL,
+	.link_slaves = NULL,
 	.unlink = NULL,
 	.writei = snd_pcm_mmap_writei,
 	.writen = snd_pcm_mmap_writen,
@@ -576,11 +580,8 @@ static snd_pcm_fast_ops_t snd_pcm_dshare_fast_ops = {
  * \brief Creates a new dshare PCM
  * \param pcmp Returns created PCM handle
  * \param name Name of PCM
- * \param ipc_key IPC key for semaphore and shared memory
- * \param ipc_perm IPC permissions for semaphore and shared memory
+ * \param opts Direct PCM configurations
  * \param params Parameters for slave
- * \param bindings Channel bindings
- * \param slowptr Slow but more precise pointer updates
  * \param root Configuration root
  * \param sconf Slave configuration
  * \param stream PCM Direction (stream)
@@ -591,10 +592,8 @@ static snd_pcm_fast_ops_t snd_pcm_dshare_fast_ops = {
  *          changed in future.
  */
 int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
-			key_t ipc_key, mode_t ipc_perm,
+			struct snd_pcm_direct_open_conf *opts,
 			struct slave_params *params,
-			snd_config_t *bindings,
-			int slowptr,
 			snd_config_t *root, snd_config_t *sconf,
 			snd_pcm_stream_t stream, int mode)
 {
@@ -617,7 +616,7 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 		goto _err_nosem;
 	}
 	
-	ret = snd_pcm_direct_parse_bindings(dshare, bindings);
+	ret = snd_pcm_direct_parse_bindings(dshare, params, opts->bindings);
 	if (ret < 0)
 		goto _err_nosem;
 		
@@ -627,8 +626,9 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 		goto _err_nosem;
 	}
 	
-	dshare->ipc_key = ipc_key;
-	dshare->ipc_perm = ipc_perm;
+	dshare->ipc_key = opts->ipc_key;
+	dshare->ipc_perm = opts->ipc_perm;
+	dshare->ipc_gid = opts->ipc_gid;
 	dshare->semid = -1;
 	dshare->shmid = -1;
 
@@ -663,11 +663,15 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	pcm->fast_ops = &snd_pcm_dshare_fast_ops;
 	pcm->private_data = dshare;
 	dshare->state = SND_PCM_STATE_OPEN;
-	dshare->slowptr = slowptr;
+	dshare->slowptr = opts->slowptr;
+	dshare->max_periods = opts->max_periods;
 	dshare->sync_ptr = snd_pcm_dshare_sync_ptr;
 
 	if (first_instance) {
-		ret = snd_pcm_open_slave(&spcm, root, sconf, stream, mode | SND_PCM_NONBLOCK);
+		/* recursion is already checked in
+		   snd_pcm_direct_get_slave_ipc_offset() */
+		ret = snd_pcm_open_slave(&spcm, root, sconf, stream,
+					 mode | SND_PCM_NONBLOCK, NULL);
 		if (ret < 0) {
 			SNDERR("unable to open slave");
 			goto _err;
@@ -686,42 +690,53 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 
 		dshare->spcm = spcm;
 		
-		ret = snd_pcm_direct_server_create(dshare);
-		if (ret < 0) {
-			SNDERR("unable to create server");
-			goto _err;
+		if (dshare->shmptr->use_server) {
+			ret = snd_pcm_direct_server_create(dshare);
+			if (ret < 0) {
+				SNDERR("unable to create server");
+				goto _err;
+			}
 		}
 
 		dshare->shmptr->type = spcm->type;
 	} else {
-		/* up semaphore to avoid deadlock */
-		snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
-		ret = snd_pcm_direct_client_connect(dshare);
-		if (ret < 0) {
-			SNDERR("unable to connect client");
-			goto _err_nosem;
-		}
+		if (dshare->shmptr->use_server) {
+			/* up semaphore to avoid deadlock */
+			snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
+			ret = snd_pcm_direct_client_connect(dshare);
+			if (ret < 0) {
+				SNDERR("unable to connect client");
+				goto _err_nosem;
+			}
 			
-		snd_pcm_direct_semaphore_down(dshare, DIRECT_IPC_SEM_CLIENT);
-		ret = snd_pcm_hw_open_fd(&spcm, "dshare_client", dshare->hw_fd, 0, 0);
-		if (ret < 0) {
-			SNDERR("unable to open hardware");
-			goto _err;
-		}
+			snd_pcm_direct_semaphore_down(dshare, DIRECT_IPC_SEM_CLIENT);
+			ret = snd_pcm_direct_open_secondary_client(&spcm, dshare, "dshare_client");
+			if (ret < 0)
+				goto _err;
+
+		} else {
+
+			ret = snd_pcm_open_slave(&spcm, root, sconf, stream,
+						 mode | SND_PCM_NONBLOCK |
+						 SND_PCM_APPEND,
+						 NULL);
+			if (ret < 0) {
+				SNDERR("unable to open slave");
+				goto _err;
+			}
+			if (snd_pcm_type(spcm) != SND_PCM_TYPE_HW) {
+				SNDERR("dshare plugin can be only connected to hw plugin");
+				ret = -EINVAL;
+				goto _err;
+			}
 		
-		spcm->donot_close = 1;
-		spcm->setup = 1;
-		spcm->buffer_size = dshare->shmptr->s.buffer_size;
-		spcm->sample_bits = dshare->shmptr->s.sample_bits;
-		spcm->channels = dshare->shmptr->s.channels;
-		spcm->format = dshare->shmptr->s.format;
-		spcm->boundary = dshare->shmptr->s.boundary;
-		spcm->info = dshare->shmptr->s.info;
-		ret = snd_pcm_mmap(spcm);
-		if (ret < 0) {
-			SNDERR("unable to mmap channels");
-			goto _err;
+			ret = snd_pcm_direct_initialize_secondary_slave(dshare, spcm, params);
+			if (ret < 0) {
+				SNDERR("unable to initialize slave");
+				goto _err;
+			}
 		}
+
 		dshare->spcm = spcm;
 	}
 
@@ -770,8 +785,7 @@ int snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 		snd_pcm_direct_semaphore_up(dshare, DIRECT_IPC_SEM_CLIENT);
  _err_nosem:
 	if (dshare) {
-		if (dshare->bindings)
-			free(dshare->bindings);
+		free(dshare->bindings);
 		free(dshare);
 	}
 	if (pcm)
@@ -845,85 +859,16 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 		       snd_config_t *root, snd_config_t *conf,
 		       snd_pcm_stream_t stream, int mode)
 {
-	snd_config_iterator_t i, next;
-	snd_config_t *slave = NULL, *bindings = NULL, *sconf;
+	snd_config_t *sconf;
 	struct slave_params params;
-	int bsize, psize, ipc_key_add_uid = 0, slowptr = 0;
-	key_t ipc_key = 0;
-	mode_t ipc_perm = 0600;
-	
+	struct snd_pcm_direct_open_conf dopen;
+	int bsize, psize;
 	int err;
-	snd_config_for_each(i, next, conf) {
-		snd_config_t *n = snd_config_iterator_entry(i);
-		const char *id;
-		if (snd_config_get_id(n, &id) < 0)
-			continue;
-		if (snd_pcm_conf_generic_id(id))
-			continue;
-		if (strcmp(id, "ipc_key") == 0) {
-			long key;
-			err = snd_config_get_integer(n, &key);
-			if (err < 0) {
-				SNDERR("The field ipc_key must be an integer type");
-				return err;
-			}
-			ipc_key = key;
-			continue;
-		}
-		if (strcmp(id, "ipc_perm") == 0) {
-			char *perm;
-			char *endp;
-			err = snd_config_get_ascii(n, &perm);
-			if (err < 0) {
-				SNDERR("The field ipc_perm must be a valid file permission");
-				return err;
-			}
-			if (isdigit(*perm) == 0) {
-				SNDERR("The field ipc_perm must be a valid file permission");
-				return -EINVAL;
-			}
-			ipc_perm = strtol(perm, &endp, 8);
-			continue;
-		}
-		if (strcmp(id, "ipc_key_add_uid") == 0) {
-			err = snd_config_get_bool(n);
-			if (err < 0) {
-				SNDERR("The field ipc_key_add_uid must be a boolean type");
-				return err;
-			}
-			ipc_key_add_uid = err;
-			continue;
-		}
-		if (strcmp(id, "slave") == 0) {
-			slave = n;
-			continue;
-		}
-		if (strcmp(id, "bindings") == 0) {
-			bindings = n;
-			continue;
-		}
-		if (strcmp(id, "slowptr") == 0) {
-			err = snd_config_get_bool(n);
-			if (err < 0) {
-				SNDERR("The field slowptr must be a boolean type");
-				return err;
-			}
-			slowptr = err;
-			continue;
-		}
-		SNDERR("Unknown field %s", id);
-		return -EINVAL;
-	}
-	if (!slave) {
-		SNDERR("slave is not defined");
-		return -EINVAL;
-	}
-	if (ipc_key_add_uid)
-		ipc_key += getuid();
-	if (!ipc_key) {
-		SNDERR("Unique IPC key is not defined");
-		return -EINVAL;
-	}
+
+	err = snd_pcm_direct_parse_open_conf(root, conf, stream, &dopen);
+	if (err < 0)
+		return err;
+
 	/* the default settings, it might be invalid for some hardware */
 	params.format = SND_PCM_FORMAT_S16;
 	params.rate = 48000;
@@ -932,7 +877,7 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 	params.buffer_time = -1;
 	bsize = psize = -1;
 	params.periods = 3;
-	err = snd_pcm_slave_conf(root, slave, &sconf, 8,
+	err = snd_pcm_slave_conf(root, dopen.slave, &sconf, 8,
 				 SND_PCM_HW_PARAM_FORMAT, 0, &params.format,
 				 SND_PCM_HW_PARAM_RATE, 0, &params.rate,
 				 SND_PCM_HW_PARAM_CHANNELS, 0, &params.channels,
@@ -950,9 +895,10 @@ int _snd_pcm_dshare_open(snd_pcm_t **pcmp, const char *name,
 
 	params.period_size = psize;
 	params.buffer_size = bsize;
-	err = snd_pcm_dshare_open(pcmp, name, ipc_key, ipc_perm, &params, bindings, slowptr, root, sconf, stream, mode);
-	if (err < 0)
-		snd_config_delete(sconf);
+
+	err = snd_pcm_dshare_open(pcmp, name, &dopen, &params,
+				  root, sconf, stream, mode);
+	snd_config_delete(sconf);
 	return err;
 }
 #ifndef DOC_HIDDEN
