@@ -27,7 +27,6 @@
  *
  */
   
-#include <dlfcn.h>
 #include "pcm_local.h"
 #include "pcm_generic.h"
 
@@ -55,13 +54,13 @@ static int snd_pcm_hooks_close(snd_pcm_t *pcm)
 	snd_pcm_hooks_t *h = pcm->private_data;
 	struct list_head *pos, *next;
 	unsigned int k;
-	int err;
+	int res = 0, err;
 
 	list_for_each_safe(pos, next, &h->hooks[SND_PCM_HOOK_TYPE_CLOSE]) {
 		snd_pcm_hook_t *hook = list_entry(pos, snd_pcm_hook_t, list);
 		err = hook->func(hook);
 		if (err < 0)
-			return err;
+			res = err;
 	}
 	for (k = 0; k <= SND_PCM_HOOK_TYPE_LAST; ++k) {
 		struct list_head *hooks = &h->hooks[k];
@@ -72,14 +71,17 @@ static int snd_pcm_hooks_close(snd_pcm_t *pcm)
 			snd_pcm_hook_remove(hook);
 		}
 	}
-	return snd_pcm_generic_close(pcm);
+	err = snd_pcm_generic_close(pcm);
+	if (err < 0)
+		res = err;
+	return res;
 }
 
 static int snd_pcm_hooks_hw_params(snd_pcm_t *pcm, snd_pcm_hw_params_t *params)
 {
 	snd_pcm_hooks_t *h = pcm->private_data;
 	struct list_head *pos, *next;
-	int err = snd_pcm_hw_params(h->gen.slave, params);
+	int err = snd_pcm_generic_hw_params(pcm, params);
 	if (err < 0)
 		return err;
 	list_for_each_safe(pos, next, &h->hooks[SND_PCM_HOOK_TYPE_HW_PARAMS]) {
@@ -95,7 +97,7 @@ static int snd_pcm_hooks_hw_free(snd_pcm_t *pcm)
 {
 	snd_pcm_hooks_t *h = pcm->private_data;
 	struct list_head *pos, *next;
-	int err = snd_pcm_hw_free(h->gen.slave);
+	int err = snd_pcm_generic_hw_free(pcm);
 	if (err < 0)
 		return err;
 	list_for_each_safe(pos, next, &h->hooks[SND_PCM_HOOK_TYPE_HW_FREE]) {
@@ -148,8 +150,8 @@ static snd_pcm_fast_ops_t snd_pcm_hooks_fast_ops = {
 	.rewind = snd_pcm_generic_rewind,
 	.forward = snd_pcm_generic_forward,
 	.resume = snd_pcm_generic_resume,
-	.link_fd = snd_pcm_generic_link_fd,
 	.link = snd_pcm_generic_link,
+	.link_slaves = snd_pcm_generic_link_slaves,
 	.unlink = snd_pcm_generic_unlink,
 	.writei = snd_pcm_generic_writei,
 	.writen = snd_pcm_generic_writen,
@@ -198,6 +200,7 @@ int snd_pcm_hooks_open(snd_pcm_t **pcmp, const char *name, snd_pcm_t *slave, int
 	pcm->private_data = h;
 	pcm->poll_fd = slave->poll_fd;
 	pcm->poll_events = slave->poll_events;
+	pcm->mmap_shadow = 1;
 	snd_pcm_link_hw_ptr(pcm, slave);
 	snd_pcm_link_appl_ptr(pcm, slave);
 	*pcmp = pcm;
@@ -304,7 +307,7 @@ static int snd_pcm_hook_add_conf(snd_pcm_t *pcm, snd_config_t *root, snd_config_
 	snd_config_t *type = NULL, *args = NULL;
 	snd_config_iterator_t i, next;
 	int (*install_func)(snd_pcm_t *pcm, snd_config_t *args) = NULL;
-	void *h;
+	void *h = NULL;
 	if (snd_config_get_type(conf) != SND_CONFIG_TYPE_COMPOUND) {
 		SNDERR("Invalid hook definition");
 		return -EINVAL;
@@ -393,16 +396,18 @@ static int snd_pcm_hook_add_conf(snd_pcm_t *pcm, snd_config_t *root, snd_config_
        _err:
 	if (type)
 		snd_config_delete(type);
-	if (err >= 0 && args && snd_config_get_string(args, &str) >= 0) {
-		err = snd_config_search_definition(root, "hook_args", str, &args);
-		if (err < 0) {
-			SNDERR("unknown hook_args %s", str);
-		} else {
-			err = install_func(pcm, args);
+	if (err >= 0) {
+		if (args && snd_config_get_string(args, &str) >= 0) {
+			err = snd_config_search_definition(root, "hook_args", str, &args);
+			if (err < 0)
+				SNDERR("unknown hook_args %s", str);
+			else
+				err = install_func(pcm, args);
 			snd_config_delete(args);
-		}
-	} else
-		err = install_func(pcm, args);
+		} else
+			err = install_func(pcm, args);
+		snd_dlclose(h);
+	}
 	if (err < 0)
 		return err;
 	return 0;
@@ -459,7 +464,7 @@ int _snd_pcm_hooks_open(snd_pcm_t **pcmp, const char *name,
 	err = snd_pcm_slave_conf(root, slave, &sconf, 0);
 	if (err < 0)
 		return err;
-	err = snd_pcm_open_slave(&spcm, root, sconf, stream, mode);
+	err = snd_pcm_open_slave(&spcm, root, sconf, stream, mode, conf);
 	snd_config_delete(sconf);
 	if (err < 0)
 		return err;
@@ -469,7 +474,7 @@ int _snd_pcm_hooks_open(snd_pcm_t **pcmp, const char *name,
 		return err;
 	}
 	if (!hooks)
-		return 0;
+		goto _done;
 	snd_config_for_each(i, next, hooks) {
 		snd_config_t *n = snd_config_iterator_entry(i);
 		const char *str;
@@ -488,6 +493,7 @@ int _snd_pcm_hooks_open(snd_pcm_t **pcmp, const char *name,
 			return err;
 		}
 	}
+ _done:
 	*pcmp = rpcm;
 	return 0;
 }
