@@ -72,7 +72,6 @@ This example shows opening a timer device and reading of timer events.
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
-#include <dlfcn.h>
 #include <signal.h>
 #include <sys/ioctl.h>
 #include "timer_local.h"
@@ -92,7 +91,7 @@ static int snd_timer_open_conf(snd_timer_t **timer,
 #ifndef PIC
 	extern void *snd_timer_open_symbols(void);
 #endif
-	void *h;
+	void *h = NULL;
 	if (snd_config_get_type(timer_conf) != SND_CONFIG_TYPE_COMPOUND) {
 		if (name)
 			SNDERR("Invalid type for TIMER %s definition", name);
@@ -171,7 +170,14 @@ static int snd_timer_open_conf(snd_timer_t **timer,
        _err:
 	if (type_conf)
 		snd_config_delete(type_conf);
-	return err >= 0 ? open_func(timer, name, timer_root, timer_conf, mode) : err;
+	if (! err) {
+		err = open_func(timer, name, timer_root, timer_conf, mode);
+		if (err < 0)
+			snd_dlclose(h);
+		else
+			(*timer)->dl_handle = h;
+	}
+	return err;
 }
 
 static int snd_timer_open_noupdate(snd_timer_t **timer, snd_config_t *root, const char *name, int mode)
@@ -242,12 +248,12 @@ int snd_timer_close(snd_timer_t *timer)
 		snd_async_handler_t *h = list_entry(timer->async_handlers.next, snd_async_handler_t, hlist);
 		snd_async_del_handler(h);
 	}
-	if ((err = timer->ops->close(timer)) < 0)
-		return err;
-	if (timer->name)
-		free(timer->name);
+	err = timer->ops->close(timer);
+	if (timer->dl_handle)
+		snd_dlclose(timer->dl_handle);
+	free(timer->name);
 	free(timer);
-	return 0;
+	return err;
 }
 
 /**
@@ -505,7 +511,7 @@ int snd_timer_info_get_card(snd_timer_info_t * info)
 const char *snd_timer_info_get_id(snd_timer_info_t * info)
 {
 	assert(info);
-	return info->id;
+	return (const char *)info->id;
 }
 
 /**
@@ -516,7 +522,7 @@ const char *snd_timer_info_get_id(snd_timer_info_t * info)
 const char *snd_timer_info_get_name(snd_timer_info_t * info)
 {
 	assert(info);
-	return info->name;
+	return (const char *)info->name;
 }
 
 
